@@ -688,8 +688,13 @@ class Client(object):
 
     def _result_from_response(self, response):
         """ Creates an EtcdResult from json dictionary """
+        raw_response = response.data
+        try:
+            res = json.loads(raw_response.decode('utf-8'))
+        except (TypeError, ValueError, UnicodeError) as e:
+            raise etcd.EtcdException(
+                'Server response was not valid JSON: %r' % e)
         try:
-            res = json.loads(response.data.decode('utf-8'))
             r = etcd.EtcdResult(**res)
             if response.status == 201:
                 r.newKey = True
@@ -697,9 +702,9 @@ class Client(object):
             return r
         except Exception as e:
             raise etcd.EtcdException(
-                'Unable to decode server response: %s' % e)
+                'Unable to decode server response: %r' % e)
 
-    def _next_server(self):
+    def _next_server(self, cause=None):
         """ Selects the next server in the list, refreshes the server list. """
         _log.debug("Selection next machine in cache. Available machines: %s",
                    self._machines_cache)
@@ -707,7 +712,8 @@ class Client(object):
             mach = self._machines_cache.pop()
         except IndexError:
             _log.error("Machines cache is empty, no machines to try.")
-            raise etcd.EtcdConnectionFailed('No more machines in the cluster')
+            raise etcd.EtcdConnectionFailed('No more machines in the cluster',
+                                            cause=cause)
         else:
             _log.info("Selected new etcd server %s", mach)
             return mach
@@ -752,7 +758,15 @@ class Client(object):
                 else:
                     raise etcd.EtcdException(
                         'HTTP method {} not supported'.format(method))
-
+                
+                # Check the cluster ID hasn't changed under us.  We use
+                # preload_content=False above so we can read the headers
+                # before we wait for the content of a watch.
+                self._check_cluster_id(response)
+                # Now force the data to be preloaded in order to trigger any
+                # IO-related errors in this method rather than when we try to
+                # access it later.
+                _ = response.data
             # urllib3 doesn't wrap all httplib exceptions and earlier versions
             # don't wrap socket errors either.
             except (urllib3.exceptions.HTTPError,
@@ -765,35 +779,18 @@ class Client(object):
                               "server.")
                     # _next_server() raises EtcdException if there are no
                     # machines left to try, breaking out of the loop.
-                    self._base_uri = self._next_server()
+                    self._base_uri = self._next_server(cause=e)
                     some_request_failed = True
                 else:
                     _log.debug("Reconnection disabled, giving up.")
                     raise etcd.EtcdConnectionFailed(
-                        "Connection to etcd failed due to %r" % e)
+                        "Connection to etcd failed due to %r" % e,
+                        cause=e
+                    )
             except:
                 _log.exception("Unexpected request failure, re-raising.")
                 raise
-
-            else:
-                # Check the cluster ID hasn't changed under us.  We use
-                # preload_content=False above so we can read the headers
-                # before we wait for the content of a long poll.
-                cluster_id = response.getheader("x-etcd-cluster-id")
-                id_changed = (self.expected_cluster_id
-                              and cluster_id is not None and
-                              cluster_id != self.expected_cluster_id)
-                # Update the ID so we only raise the exception once.
-                old_expected_cluster_id = self.expected_cluster_id
-                self.expected_cluster_id = cluster_id
-                if id_changed:
-                    # Defensive: clear the pool so that we connect afresh next
-                    # time.
-                    self.http.clear()
-                    raise etcd.EtcdClusterIdChanged(
-                        'The UUID of the cluster changed from {} to '
-                        '{}.'.format(old_expected_cluster_id, cluster_id))
-
+            
         if some_request_failed:
             if not self._use_proxies:
                 # The cluster may have changed since last invocation
@@ -801,6 +798,24 @@ class Client(object):
             self._machines_cache.remove(self._base_uri)
         return self._handle_server_response(response)
 
+    def _check_cluster_id(self, response):
+        cluster_id = response.getheader("x-etcd-cluster-id")
+        if not cluster_id:
+            _log.warning("etcd response did not contain a cluster ID")
+            return
+        id_changed = (self.expected_cluster_id and
+                      cluster_id != self.expected_cluster_id)
+        # Update the ID so we only raise the exception once.
+        old_expected_cluster_id = self.expected_cluster_id
+        self.expected_cluster_id = cluster_id
+        if id_changed:
+            # Defensive: clear the pool so that we connect afresh next
+            # time.
+            self.http.clear()
+            raise etcd.EtcdClusterIdChanged(
+                'The UUID of the cluster changed from {} to '
+                '{}.'.format(old_expected_cluster_id, cluster_id))
+
     def _handle_server_response(self, response):
         """ Handles the server response """
         if response.status in [200, 201]:
