@@ -134,7 +134,7 @@ class Lock(object):
         return self.path + '/' + str(self._sequence)
 
     def _set_sequence(self, key):
-        self._sequence = int(key.replace(self.path, '').lstrip('/'))
+        self._sequence = key.replace(self.path, '').lstrip('/')
 
     def _find_lock(self):
         if self._sequence:
